use std::{
    env::{self, VarError},
    error::Error,
    fmt::Debug,
    path::PathBuf,
    str::FromStr,
    time::Duration,
};

use anyhow::{anyhow, Context};

pub struct Config {
    pub capture_images: bool,
    pub capture_audio_duration: Option<f64>,
    pub shows_file: PathBuf,
    pub global_tags: Vec<String>,
    pub post_interval: Duration,
    pub cohost_email: String,
    pub cohost_password: String,
    pub cohost_page: String,
    pub cohost_draft: bool,
    pub cohost_cw: bool,
    pub eighteen_plus: bool,
}

const VAR_PREFIX: &str = "SCREENCAP_BOT_";

fn get_var(name: &str) -> anyhow::Result<Option<String>> {
    let var_name = VAR_PREFIX.to_string() + name;
    match env::var(&var_name) {
        Ok(v) => Ok(Some(v)),
        Err(VarError::NotPresent) => Ok(None),
        Err(e) => Err(e).with_context(|| format!("Failed to read variable {}", var_name)),
    }
}

fn parse_var<E: Debug + Send + Sync + Error + 'static, T: FromStr<Err = E>>(
    name: &str,
) -> anyhow::Result<Option<T>> {
    get_var(name)?
        .map(|s| {
            s.parse()
                .with_context(|| format!("Failed to parse variable {}{}", VAR_PREFIX, name))
        })
        .transpose()
}

fn require_var(name: &str) -> anyhow::Result<String> {
    get_var(name)?.ok_or_else(|| anyhow!("{}{} must be set", VAR_PREFIX, name))
}

pub fn load() -> anyhow::Result<Config> {
    let capture_images = parse_var("CAPTURE_IMAGES")?.unwrap_or(true);

    let capture_audio_duration = parse_var("CAPTURE_AUDIO_DURATION")?;
    if let Some(d) = capture_audio_duration {
        if d <= 0.0 {
            return Err(anyhow!(
                "{}CAPTURE_AUDIO_DURATION cannot be <= 0",
                VAR_PREFIX
            ));
        }
    }

    if let (false, None) = (capture_images, capture_audio_duration) {
        return Err(anyhow!(
            "At least one of image capture and audio capture must be enabled!"
        ));
    }

    Ok(Config {
        capture_images,
        capture_audio_duration,
        shows_file: parse_var("SHOWS_FILE")?.unwrap_or_else(|| PathBuf::from("./shows.yaml")),
        global_tags: get_var("GLOBAL_TAGS")?
            .map(|s| s.split(',').map(String::from).collect())
            .unwrap_or_default(),
        post_interval: Duration::from_secs(parse_var("POST_INTERVAL")?.unwrap_or_default()),
        cohost_email: require_var("COHOST_EMAIL")?,
        cohost_password: require_var("COHOST_PASSWORD")?,
        cohost_page: require_var("COHOST_PAGE")?,
        cohost_draft: parse_var("COHOST_DRAFT")?.unwrap_or(false),
        cohost_cw: parse_var("COHOST_CW")?.unwrap_or(capture_images),
        eighteen_plus: parse_var("18PLUS")?.unwrap_or(false),
    })
}
